#! /bin/sh

#
# @(#) Installation of HPU as a stored procedure
#


#
# 
#


# set -x
set -u


    
AWK=awk
CAT=cat
CHGRP=chgrp
CHMOD=chmod
CHOWN=chown
COMM=comm
CP=cp
CUT=cut
DF=df
EXPR=expr
FIND=find
GREP=grep
ID=id
KILL=kill
LN=ln
LS=ls
MKDIR=mkdir
MORE=more
MV=mv
PRINTF=printf
PS=ps
RM=rm
RMDIR=rmdir
SED=sed
SORT=sort
TAR=tar
TR=tr
UNAME=uname
WC=wc
XARGS=xargs


###########################################################################
# DisplayTrace
###########################################################################

# 
# Affichage message de trace (sur sortie standard)
# 
# 
# Argument 1 : texte du message
# 
# 

DisplayTrace ()
{
    # RETURN_CODE=0
    
    echo "$1"
    
    # return ${RETURN_CODE}
}




###########################################################################
# DisplayError
###########################################################################

# 
# Affichage message sur stderr
# 
# 
# Argument 1 : texte du message
# 
# 
# 

DisplayError ()
{
    # RETURN_CODE=0
    
    echo "$1" 1>&2
    
    # return ${RETURN_CODE}
}


###########################################################################
# CheckExistingDirectory
###########################################################################

# Verification si repertoire existant
#
# Pas de verification si droits en ecriture !
#

CheckExistingDirectory ()
{
    RETURN_CODE=0
    
    
    if [ ! -d "$1" ]
    then
        echo "Error checking existing directory \"$1\"." 1>&2
        RETURN_CODE=1
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# CheckReadableFile
###########################################################################

# Verification si fichier existant et lisible
#

CheckReadableFile ()
{
    RETURN_CODE=0
    

    # DisplayTrace ""
    # DisplayTrace ">>> CheckReadableFile"

    
    if [ ! -r $1 ]
    then
        echo "Error checking readable file \"$1\"." 1>&2
        RETURN_CODE=1
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# GetDB2InstanceBitWidth
###########################################################################

GetDB2InstanceBitWidth ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> GetDB2InstanceBitWidth"
    

    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    
        if [ -z "$1" ]
        then
            echo "Empty argument" 1>&2
            echo '\07Error'
            RETURN_CODE=2
        else
            FILE_INTERNAL_INSTANCE=$1/sqllib/ctrl/.instuse

            CheckReadableFile "${FILE_INTERNAL_INSTANCE}" 1>/dev/null 2>&1
            RETURN_CODE=$?

            if [ ${RETURN_CODE} -eq 0 ]
            then
                LINE_BITWIDTH=`${GREP} "^BitWidth" "${FILE_INTERNAL_INSTANCE}"`
                RETURN_CODE=$?
            fi    
            
            if [ ${RETURN_CODE} -eq 0 ]
            then
                echo "${LINE_BITWIDTH}" | ${CUT} -d "=" -f2
            fi

        fi

    fi    
    
    return ${RETURN_CODE}
}


###########################################################################
# main
###########################################################################

# Argument 1 : HPU installation directory
# Argument 2 : database name
# Argument 3 : 
#



RETURN_CODE=0


# Check arguments 

if [ $# -ne 2 ]
then
    echo "Incorrect number of arguments" 1>&2
    echo "`basename $0` HPU_installation_directory database_name" 1>&2
    exit 1
fi


DIR_HPU_INSTALLATION="$1"
DATABASE_NAME="$2"


if [ "${RETURN_CODE}" -eq 0 ]
then
    CheckExistingDirectory "${DIR_HPU_INSTALLATION}"
    RETURN_CODE=$?
fi



if [ "${RETURN_CODE}" -eq 0 ]
then
    DIR_INSTANCE_HOME="${HOME}"

    CheckExistingDirectory "${DIR_INSTANCE_HOME}"
    RETURN_CODE=$?
fi




# Check instance bitwidth

if [ ${RETURN_CODE} -eq 0 ]
then
    HPU_BIT_WIDTH=`GetDB2InstanceBitWidth ${DIR_INSTANCE_HOME}`
    RETURN_CODE=$?
    if [ ${RETURN_CODE} -ne 0 ]
    then
        DisplayError "Error while retrieving instance bit width"
    fi
fi    


if [ ${RETURN_CODE} -eq 0 ]
then
    case "${HPU_BIT_WIDTH}" in
    32|64)
        echo "Instance is ${HPU_BIT_WIDTH}-bit"
        ;;

    *)
        DisplayError "Incorrect value for HPU_BIT_WIDTH: \"${HPU_BIT_WIDTH}\" " 1>&2 
        RETURN_CODE=1
        ;;
    esac    
fi


# Check destination directory

if [ "${RETURN_CODE}" -eq 0 ]
then
    DIR_FUNCTIONS="${DIR_INSTANCE_HOME}/sqllib/function"

    CheckExistingDirectory "${DIR_FUNCTIONS}"
    RETURN_CODE=$?
fi






if [ "${RETURN_CODE}" -eq 0 ]
then
    case "${HPU_BIT_WIDTH}" in
    32)
        FILE_PROCEDURE="db2hpuproc.so"
        ;;

    64)
        FILE_PROCEDURE="db2hpuproc64.so"
        ;;
        
    *)
        DisplayError "Incorrect value for HPU_BIT_WIDTH: \"${HPU_BIT_WIDTH}\" " 1>&2 
        RETURN_CODE=1
        ;;
    esac    
fi


# Check source directory

if [ "${RETURN_CODE}" -eq 0 ]
then
    DIR_SOURCE_PROCEDURE="${DIR_HPU_INSTALLATION}/install/function"

    CheckExistingDirectory "${DIR_SOURCE_PROCEDURE}"
    RETURN_CODE=$?
fi




# Check file

if [ "${RETURN_CODE}" -eq 0 ]
then
    CheckReadableFile "${DIR_HPU_INSTALLATION}/install/function/${FILE_PROCEDURE}"
    RETURN_CODE=$?
fi


if [ "${RETURN_CODE}" -eq 0 ]
then
    echo "Copy file \"${FILE_PROCEDURE}\" to stored procedures directory"
    \cp "${DIR_HPU_INSTALLATION}/install/function/${FILE_PROCEDURE}" \
        "${DIR_FUNCTIONS}"
    if [ $? -ne 0 ]
    then
        DisplayError "Error while copying file \"${DIR_HPU_INSTALLATION}/install/function/${FILE_PROCEDURE}\"  "
        DisplayError "to \"${DIR_FUNCTIONS}\" "
        RETURN_CODE=1
    fi
fi





# Add stored procedure to DB2

DB_CONNECTION=0  

if [ "${RETURN_CODE}" -eq 0 ]
then
    echo "Connection to database \"${DATABASE_NAME}\" "
    \db2 +o +p -t << EOF1
CONNECT TO ${DATABASE_NAME} ; 
EOF1
    RETURN_CODE=$?
    if [ "${RETURN_CODE}" -ne 0 ]
    then
        DisplayError "Error while connecting to database \"${DATABASE_NAME}\" "
        DB_CONNECTION=0  
    else
        DB_CONNECTION=1
    fi
    
fi



if [ "${RETURN_CODE}" -eq 0 ]
then
    echo "Creation of stored procedure in database \"${DATABASE_NAME}\" "
    EXTERNAL_NAME="${FILE_PROCEDURE}!call_db2hpu"
    
    \db2 +o +p -t << EOF2
CREATE PROCEDURE DB2HPU (IN tophpu VARCHAR(512), IN cmdline CLOB(1024), IN control CLOB(64K), OUT stderr CLOB(64K), OUT rc INTEGER)
DYNAMIC RESULT SETS 0
LANGUAGE C PARAMETER STYLE SQL
DBINFO
FENCED
THREADSAFE
INHERIT SPECIAL REGISTERS
EXTERNAL NAME '${EXTERNAL_NAME}' ;
EOF2
    RETURN_CODE=$?
    if [ "${RETURN_CODE}" -ne 0 ]
    then
        DisplayError "Error while creating stored procedure in database \"${DATABASE_NAME}\" "
    fi
fi


if [ "${DB_CONNECTION}" -eq 1 ]
then
    echo "Disconnection from database \"${DATABASE_NAME}\" "
    \db2 +o +p -t << EOF3
CONNECT RESET ;
EOF3
    RETURN_CODE=$?
    if [ "${RETURN_CODE}" -ne 0 ]
    then
        DisplayError "Error while disconnecting from database \"${DATABASE_NAME}\" "
    fi
fi


if [ "${DB_CONNECTION}" -eq 1 ]
then
    echo "Ending of database use"
    \db2 +o +p -t << EOF3
TERMINATE ;
EOF3
    RETURN_CODE=$?
    if [ "${RETURN_CODE}" -ne 0 ]
    then
        DisplayError "Error while ending database use "
    fi
fi

### DEBUG ###
### exit
### FIN DEBUG ### 


exit ${RETURN_CODE}


